/*
 * Entagged Audio Tag library
 * Copyright (c) 2003-2005 Raphal Slinckx <raphael@slinckx.net>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package entagged.audioformats.mp3.util.id3frames;

import java.io.UnsupportedEncodingException;

import entagged.audioformats.generic.TagField;
import entagged.audioformats.generic.TagTextField;

public class CommId3Frame extends TextId3Frame implements TagTextField {
	
	private String shortDesc;
	private String lang;
	
	/*
	 * 0,1| frame flags
	 * 2| encoding
	 * 3,4,5| lang
	 * 6,..,0x00(0x00)| short descr
	 * x,..| actual comment
	 */
	
	public CommId3Frame(String content) {
		super("COMM", content);
		
		this.shortDesc = "";
		this.lang = "eng";
	}
	
	public CommId3Frame(byte[] rawContent, byte version) throws UnsupportedEncodingException {
		super("COMM", rawContent, version);
	}
	
	public String getLangage() {
		return this.lang;
	}
	
	protected void populate(byte[] raw) throws UnsupportedEncodingException {
		this.encoding = raw[flags.length];
		
		this.lang = new String(raw, flags.length+1, 3);
		
		this.shortDesc = getString(raw, flags.length+4, raw.length - flags.length - 4, getEncoding());
		
		int i = shortDesc.indexOf("\u0000");
		if(i == -1) {
			this.content = "";
			return;
		}
		
		this.content = this.shortDesc.substring(i+1);
		this.shortDesc = this.shortDesc.substring(0, i);
	}
	
	protected byte[] build() throws UnsupportedEncodingException {
		String text = this.shortDesc + "\u0000" + this.content;
		byte[] data = getBytes(text, getEncoding());
		byte[] lan = this.lang.getBytes();
		
		//the return byte[]
		byte[] b = new byte[4 + 4 + flags.length + 1 + 3 + data.length];
		
		int offset = 0;
		copy(getIdBytes(), b, offset);        offset += 4;
		copy(getSize(b.length-10), b, offset); offset += 4;
		copy(flags, b, offset);               offset += flags.length;
		
		b[offset] = this.encoding;	offset += 1;
		
		copy(lan, b, offset);		offset += lan.length;
		
		copy(data, b, offset);
		
		return b;
	}
	
	public String getShortDescription() {
		return shortDesc;
	}
	
	public boolean isEmpty() {
	    return this.content.equals("") && this.shortDesc.equals("");
	}
	
	public void copyContent(TagField field) {
	    super.copyContent(field);
	    if(field instanceof CommId3Frame) {
	        this.shortDesc = ((CommId3Frame)field).getShortDescription();
	        this.lang = ((CommId3Frame)field).getLangage();
	    }
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("[").append(getLangage()).append("] ").append("(").append(getShortDescription()).append(") ").append(getContent());
		return sb.toString();
	}
	
	/*
	public static void main(String[] args) throws Exception {
		byte[][] content = {
				{0x00, 0x00, 0x00, 'e','n','g', 'S','h',0x00,'C','o'},
				{0x00, 0x00, 0x00, 'e','n','g', 'S','h',0x00},
				{0x00, 0x00, 0x00, 'e','n','g', 'S','h'},
				{0x00, 0x00, 0x01, 'e','n','g', (byte)0xFE, (byte)0xFF,0x00,'S',0x00,'h',0x00,0x00,0x00,'T'},
			};
		
		for(int i = 0; i<content.length; i++) {
			CommId3Frame t = new CommId3Frame(content[i], Id3v2Tag.ID3V23);
			System.out.println("-------");
			System.out.println(t.isBinary());
			System.out.println(t.getContent());
			System.out.println(t.getLangage());
			System.out.println(t.getShortDescription());
			System.out.println(t.getEncoding());
			System.out.println(t.getId());
		
		
			byte[] bytes = t.build();
			for(int j = 0; j<bytes.length; j++)
				System.out.print(Integer.toHexString(bytes[j]&0xFF)+"|");
		}
	}
	*/
}
